﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using ff14bot;

namespace Zekken
{
    public class SpellCapture
    {
        private object locker = new object();
        private const uint WM_KEYDOWN = 0x100;
        private const uint WM_KEYUP = 0x0101;
        private IntPtr windowHandle;
        private volatile bool uiHidden;
        private const int WAIT_BEFORE_SCREENSHOT = 250;

        public SpellCapture()
        {
            FileTools.PrepareDirectory(Directories.DEFAULT_SCREENSHOT_DIR);
            windowHandle = Core.Memory.Process.MainWindowHandle;
        }

        public void Capture(uint spellId, string spellname)
        {
            var name = string.Format("{0} (ID: {1})", spellname, spellId);
            Task.Run(() => TakeScreenshot(name));
        }

        private void TakeScreenshot(string name)
        {
            HideOrShowUi(true);
            Thread.Sleep(WAIT_BEFORE_SCREENSHOT);

            Bitmap image;
            try { image = PrintWindow(windowHandle); }
            catch { return; }

            HideOrShowUi(false);

            var path = Path.Combine(Directories.DEFAULT_SCREENSHOT_DIR, name) + ".png";
            if (!FileTools.PrepareSave(path))
            {
                Logger.WriteMessage("Could not prepare save for screenshot file.");
                return;
            }

            try
            {
                using (var file = new FileStream(path, FileMode.Create))
                {
                    image.Save(file, ImageFormat.Png);
                }
            }
            catch
            {
                Logger.WriteMessage("Could not save screenshot file.");
                return;
            }

            Logger.WriteMessage("Took screenshot of unknown spell {0}.", name);
        }

        private void HideOrShowUi(bool hide)
        {
            lock (locker)
            {
                if (uiHidden && hide) { return; }

                PostMessage(windowHandle, WM_KEYDOWN, (IntPtr)(Keys.Scroll), IntPtr.Zero);
                PostMessage(windowHandle, WM_KEYUP, (IntPtr)(Keys.Scroll), IntPtr.Zero);

                uiHidden = hide;
            }
        }

        [DllImport("user32.dll")]
        private static extern bool GetWindowRect(IntPtr hWnd, out WindowRectangle lpRect);
        [DllImport("user32.dll")]
        private static extern bool PrintWindow(IntPtr hWnd, IntPtr hdcBlt, int nFlags);
        [DllImport("user32.dll")]
        private static extern IntPtr PostMessage(IntPtr hWnd, uint msg, IntPtr wParam, IntPtr lParam);

        private static Bitmap PrintWindow(IntPtr hwnd)
        {
            WindowRectangle rc;
            GetWindowRect(hwnd, out rc);

            var bmp = new Bitmap(rc.Width, rc.Height, PixelFormat.Format16bppRgb555);
            var gfxBmp = Graphics.FromImage(bmp);
            var hdcBitmap = gfxBmp.GetHdc();

            PrintWindow(hwnd, hdcBitmap, 0);

            gfxBmp.ReleaseHdc(hdcBitmap);
            gfxBmp.Dispose();

            return bmp;
        }
    }
}
